/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#pragma once

#include "vlccommon.hxx"
#include <cppuhelper/compbase.hxx>
#include <com/sun/star/media/XPlayer.hpp>
#include <cppuhelper/basemutex.hxx>

#include <wrapper/Instance.hxx>
#include <wrapper/Media.hxx>
#include <wrapper/Player.hxx>
#include <wrapper/EventManager.hxx>

namespace avmedia::vlc {

typedef ::cppu::WeakComponentImplHelper< css::media::XPlayer,
                                         css::lang::XServiceInfo > VLC_Base;

class VLCPlayer : public ::cppu::BaseMutex,
                  public VLC_Base
{
    wrapper::EventHandler& mEventHandler;

    wrapper::Media         mMedia;
    wrapper::Player        mPlayer;
    wrapper::EventManager  mEventManager;
    const OUString         mUrl;
    bool                   mPlaybackLoop;
    css::uno::Reference< css::media::XFrameGrabber > mrFrameGrabber;
    intptr_t               mPrevWinID;
public:
    VLCPlayer( const OUString& url,
               wrapper::Instance& instance,
               wrapper::EventHandler& eh );

    void setVideoSize( unsigned width, unsigned height );

    void setWindowID( const intptr_t windowID );

    void SAL_CALL start() override;
    void SAL_CALL stop() override;
    sal_Bool SAL_CALL isPlaying() override;
    double SAL_CALL getDuration() override;
    void SAL_CALL setMediaTime( double fTime ) override;
    double SAL_CALL getMediaTime() override;
    void SAL_CALL setPlaybackLoop( sal_Bool bSet ) override;
    sal_Bool SAL_CALL isPlaybackLoop() override;
    void SAL_CALL setVolumeDB( ::sal_Int16 nDB ) override;
    ::sal_Int16 SAL_CALL getVolumeDB() override;
    void SAL_CALL setMute( sal_Bool bSet ) override;
    sal_Bool SAL_CALL isMute() override;
    css::awt::Size SAL_CALL getPreferredPlayerWindowSize() override;
    css::uno::Reference< css::media::XPlayerWindow > SAL_CALL createPlayerWindow( const css::uno::Sequence< css::uno::Any >& aArguments ) override;
    css::uno::Reference< css::media::XFrameGrabber > SAL_CALL createFrameGrabber() override;

    OUString SAL_CALL getImplementationName() override;
    sal_Bool SAL_CALL supportsService( const OUString& serviceName ) override;
    css::uno::Sequence< OUString > SAL_CALL getSupportedServiceNames() override;

private:
    void replay();
};

}


/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
